
;
; SMS Palette 1.0
; by Omar Cornut (Bock) on the 26 of July 2001
;

.INCLUDE "sms.inc"

.EMPTYFILL $00
.SMSTAG
.COMPUTESMSCHECKSUM

.MEMORYMAP
        DEFAULTSLOT     0
        SLOTSIZE        PAGE_SIZE
        SLOT            0               PAGE_0
        SLOT            1               PAGE_1
.ENDME

.ROMBANKMAP
        BANKSTOTAL      2
        BANKSIZE        PAGE_SIZE
        BANKS           2
.ENDRO

.BANK 0 SLOT 0

; Start -----------------------------------------------------------------------
.ORGA   $0000
        di
        im      1
        ld      sp,     $DFF0
        jp      start
;------------------------------------------------------------------------------

; Tools ---------------------------------------------------------------------
.ORGA   $0010
vdp_write_de:
                ld      a, e
                out     (VDP_ADDR), a
                ld      a, d
                out     (VDP_ADDR), a
                ret
.ORGA   $0018
vdp_write_addr_de:
                ld      a, e
                out     (VDP_ADDR), a
                ld      a, d
                or      $40
                out     (VDP_ADDR), a
                ret

.ORGA   $0028
vdp_write_addr_hl:
                ld      a, l
                out     (VDP_ADDR), a
                ld      a, h
                or      $40
                out     (VDP_ADDR), a
                ret

; Interrupt -------------------------------------------------------------------
.ORGA   $0038
interrupt:
        di
        in      a, (VDP_STATUS)
        and     $80
        jr      z, interrupt_handle
interrupt_end:
        ei
        ret

;------------------------------------------------------------------------------

; NMI -------------------------------------------------------------------------
.ORGA   $0066
        reti
;------------------------------------------------------------------------------

interrupt_handle:
        in      a, (PORT_VLINE)

;        cp      47
;        jr      z, interrupt_display_disable
        cp      49
        jr      z, interrupt_ps1
;        cp      51
;        jr      z, interrupt_display_enable

;        cp      87
;        jr      z, interrupt_display_disable
        cp      89
        jr      z, interrupt_ps2
;        cp      91
;        jr      z, interrupt_display_enable

;        cp      127
;        jr      z, interrupt_display_disable
        cp      129
        jr      z, interrupt_ps3
;        cp      131
;        jr      z, interrupt_display_enable

;        cp      160
;        jr      z, interrupt_display_disable
        cp      169
        jr      z, interrupt_ps0
;        cp      175
;        jr      z, interrupt_display_enable

        jp      interrupt_end

interrupt_display_disable:
        ld      de, $8180               ; Disable display
        rst     $10
        jp      interrupt_end

interrupt_display_enable:
        ld      de, $81C0               ; Enable display
        rst     $10
        jp      interrupt_end

interrupt_ps0:
        ld      c, 0
        call    palette_set
        jp      interrupt_end

interrupt_ps1:
        ld      c, 16
        call    palette_set
        jp      interrupt_end

interrupt_ps2:
        ld      c, 32
        call    palette_set
        jp      interrupt_end

interrupt_ps3:
        ld      c, 48
        call    palette_set
        jp      interrupt_end

.INCLUDE "vdp.asm"

start:
        call    vdp_init

        ld      bc, VRAM_TILE_SIZE * 16
        ld      hl, tiles_data
        ld      de, $0000
        call    vdp_load_data

        ld      bc, VRAM_TILE_SIZE * 10
        ld      hl, font_data
        ld      de, $0200
        call    vdp_load_data

        ld      bc, VRAM_TILE_SIZE * 16
        ld      hl, title_data
        ld      de, $0400
        call    vdp_load_data

        call    palette_set_init

        ld      c, 0
        call    palette_set

        call    tiles_map_make

        ld      de, $81C0               ; Enable display
        rst     $10
        ei

loop:
;        halt
        jp      loop

; Palette_Set_Init ------------------------------------------------------------
palette_set_init:
        ld      a, 16
        out     (VDP_ADDR), a
        ld      a, %11000000
        out     (VDP_ADDR), a
        ld      a, %00000000
        out     (VDP_DATA), a
        ld      a, %00111111
        out     (VDP_DATA), a
        ret

; Palette_Set -----------------------------------------------------------------
; In: C = Starting color (0 | 16 | 32 | 48)
palette_set:
        ld      a, 0
        out     (VDP_ADDR), a
        ld      a, %11000000
        out     (VDP_ADDR), a
        ld      a, c
        ld      b, 16
palette_set_loop:
        out     (VDP_DATA), a
        inc     a
        djnz    palette_set_loop
        ret
;------------------------------------------------------------------------------

tiles_map_make:
        ld      de,     $3800|(0*VRAM_TILE_SIZE*2)
        rst     $18

        ld      c,      $08
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_title
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line

        ld      c,      $00
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line

        ld      c,      $08
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_numbers_00_to_15
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line

        ld      c,      $00
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line

        ld      c,      $08
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_numbers_16_to_31
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line

        ld      c,      $00
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line

        ld      c,      $08
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_numbers_32_to_47
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line

        ld      c,      $00
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_colors
        call    tiles_map_make_line

        ld      c,      $08
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_numbers_48_to_63
        call    tiles_map_make_line
        ld      hl,     tiles_map_line_empty
        call    tiles_map_make_line

        ret

tiles_map_make_line:
        ld      b, 32
tiles_map_make_line_loop:
        ld      a, (hl)
        inc     hl
        out     (VDP_DATA), a
        ld      a, c
        out     (VDP_DATA), a
        djnz    tiles_map_make_line_loop
        ret

;------------------------------------------------------------------------------
tiles_map_line_empty:
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
tiles_map_line_title:
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
.DB     $20,  $21,  $22,  $23,  $24,  $25,  $26,  $27,
.DB     $28,  $29,  $2A,  $2B,  $2C,  $2D,  $2E,  $2F,
.DB     $00,  $00,  $00,  $00,  $00,  $00,  $00,  $00,
tiles_map_line_colors:
.DB     $00,  $00,  $01,  $01,  $02,  $02,  $03,  $03,
.DB     $04,  $04,  $05,  $05,  $06,  $06,  $07,  $07,
.DB     $08,  $08,  $09,  $09,  $0A,  $0A,  $0B,  $0B,
.DB     $0C,  $0C,  $0D,  $0D,  $0E,  $0E,  $0F,  $0F,
tiles_map_line_numbers_00_to_15:
.DB     $00,  $10,  $00,  $11,  $00,  $12,  $00,  $13,
.DB     $00,  $14,  $00,  $15,  $00,  $16,  $00,  $17,
.DB     $00,  $18,  $00,  $19,  $11,  $10,  $11,  $11,
.DB     $11,  $12,  $11,  $13,  $11,  $14,  $11,  $15,
tiles_map_line_numbers_16_to_31:
.DB     $11,  $16,  $11,  $17,  $11,  $18,  $11,  $19,
.DB     $12,  $10,  $12,  $11,  $12,  $12,  $12,  $13,
.DB     $12,  $14,  $12,  $15,  $12,  $16,  $12,  $17,
.DB     $12,  $18,  $12,  $19,  $13,  $10,  $13,  $11,
tiles_map_line_numbers_32_to_47:
.DB     $13,  $12,  $13,  $13,  $13,  $14,  $13,  $15,
.DB     $13,  $16,  $13,  $17,  $13,  $18,  $13,  $19,
.DB     $14,  $10,  $14,  $11,  $14,  $12,  $14,  $13,
.DB     $14,  $14,  $14,  $15,  $14,  $16,  $14,  $17,
tiles_map_line_numbers_48_to_63:
.DB     $14,  $18,  $14,  $19,  $15,  $10,  $15,  $11,
.DB     $15,  $12,  $15,  $13,  $15,  $14,  $15,  $15,
.DB     $15,  $16,  $15,  $17,  $15,  $18,  $15,  $19,
.DB     $16,  $10,  $16,  $11,  $16,  $12,  $16,  $13,
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
tiles_data:
.INCLUDE "tiles.inc"
font_data:
.INCLUDE "font.inc"
title_data:
.INCLUDE "title.inc"
;------------------------------------------------------------------------------

.BANK 1 SLOT 1

; CHECKSUM --------------------------------------------------------------------
.ORGA   $7FF0

	.DB	"TMR SEGA"	; Trademark
        .DW     $0120           ; Year
	.DW	$0000		; Checksum not correct
	.DW	$0000		; Part Num not correct
        .DW     $2607           ; Day / Month

